/*
*	Towers of Hanoi
*	by Scott Haywood
*
*	Updated 22 January 2001
*
*   This program demonstrates the use of a recursive function
*   to solve the Towers of Hanoi problem.
*
*   Tower 1 starts out with n disks 
*   arranged by size with the largest on the bottom.
*   Towers 2 and 3 start out empty.
*   The program moves one disk at a time, from the top
*	of one tower to the top of another,
*   until all disks are in order on tower 2.
*   At no time is a larger disk placed on a smaller one.
*
*	In this version, a description of each move (in english)
*	is written to standard output.
*	The towers have single-character names.
*	The program has no internal representation of the disks.
*/

#include <iostream>
#include <cassert>
#include <cstdlib>

// Move n disks from src to dest,
// using tmp as temporary storage.
// Return the number of times a single disk is moved.
//
int move(int n, char src, char dest, char tmp)
{
	static int nmoves = 0;
	if (n == 1) {
	    cout <<  "move disk from " << src << " to " << dest << '\n';
		nmoves++;
	} else {
		move(n-1, src, tmp, dest);
		move(1, src, dest, tmp);
		move(n-1, tmp, dest, src);
	}
	return nmoves;
}

int main(int argc, char** argv)
{
	int n; // number of disks

	if (argc > 1) {
		n = atoi(argv[1]);
	} else {
		cout << "\nEnter the number of disks: " ;
		cin >> n;
		cout << endl;
	}
	assert(n>0 && n<100);

	// move disks
	//
	int nmoves = move(n, '1', '2', '3');

	// summarize
	//
	cout << n << " disk" << (n==1?"":"s") << ", " 
		<< nmoves << " move" << (nmoves==1?"":"s") << "\n\n";
}
